/* $Id: input.c,v 1.7.2.1 2003/02/01 14:33:31 cegger Exp $
******************************************************************************

   Linux evdev inputlib

   Copyright (C) 2000 Marcus Sundberg	[marcus@ggi-project.org]
   Copyright (C) 2001 Stefan Seefeld	[stefan@berlin-consortium.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include "config.h"
#include "linux_evdev.h"
#include <ggi/internal/gii.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>

#define SKIPWHITE(str)	while (isspace((int)*(str))) (str)++

char *events[EV_MAX + 1] = { "Reset", "Key", "Relative", "Absolute", NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
NULL, NULL, NULL, "LED", "Sound", NULL, "Repeat" };
char *keys[KEY_MAX + 1] = { "Reserved", "Esc", "1", "2", "3", "4", "5", "6", "7", "8", "9", "0", "Minus", "Equal", "Backspace",
"Tab", "Q", "W", "E", "R", "T", "Y", "U", "I", "O", "P", "LeftBrace", "RightBrace", "Enter", "LeftControl", "A", "S", "D", "F", "G",
"H", "J", "K", "L", "Semicolon", "Apostrophe", "Grave", "LeftShift", "BackSlash", "Z", "X", "C", "V", "B", "N", "M", "Comma", "Dot",
"Slash", "RightShift", "KPAsterisk", "LeftAlt", "Space", "CapsLock", "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10",
"NumLock", "ScrollLock", "KP7", "KP8", "KP9", "KPMinus", "KP4", "KP5", "KP6", "KPPlus", "KP1", "KP2", "KP3", "KP0", "KPDot", "103rd",
"F13", "102nd", "F11", "F12", "F14", "F15", "F16", "F17", "F18", "F19", "F20", "KPEnter", "RightCtrl", "KPSlash", "SysRq",
"RightAlt", "LineFeed", "Home", "Up", "PageUp", "Left", "Right", "End", "Down", "PageDown", "Insert", "Delete", "Macro", "Mute",
"VolumeDown", "VolumeUp", "Power", "KPEqual", "KPPlusMinus", "Pause", "F21", "F22", "F23", "F24", "KPComma", "LeftMeta", "RightMeta",
"Compose", "Stop", "Again", "Props", "Undo", "Front", "Copy", "Open", "Paste", "Find", "Cut", "Help", "Menu", "Calc", "Setup",
"Sleep", "WakeUp", "File", "SendFile", "DeleteFile", "X-fer", "Prog1", "Prog2", "WWW", "MSDOS", "Coffee", "Direction",
"CycleWindows", "Mail", "Bookmarks", "Computer", "Back", "Forward", "CloseCD", "EjectCD", "EjectCloseCD", "NextSong", "PlayPause",
"PreviousSong", "StopCD", "Record", "Rewind", "Phone", "ISOKey", "Config", "HomePage", "Refresh", "Exit", "Move", "Edit", "ScrollUp",
"ScrollDown", "KPLeftParenthesis", "KPRightParenthesis",
"International1", "International2", "International3", "International4", "International5",
"International6", "International7", "International8", "International9",
"Language1", "Language2", "Language3", "Language4", "Language5", "Language6", "Language7", "Language8", "Language9",
NULL, 
"PlayCD", "PauseCD", "Prog3", "Prog4", "Suspend", "Close",
NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
"Btn0", "Btn1", "Btn2", "Btn3", "Btn4", "Btn5", "Btn6", "Btn7", "Btn8", "Btn9",
NULL, NULL,  NULL, NULL, NULL, NULL,
"LeftBtn", "RightBtn", "MiddleBtn", "SideBtn", "ExtraBtn", "ForwardBtn", "BackBtn",
NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
"Trigger", "ThumbBtn", "ThumbBtn2", "TopBtn", "TopBtn2", "PinkieBtn",
"BaseBtn", "BaseBtn2", "BaseBtn3", "BaseBtn4", "BaseBtn5", "BaseBtn6",
NULL, NULL, NULL, NULL,
"BtnA", "BtnB", "BtnC", "BtnX", "BtnY", "BtnZ", "BtnTL", "BtnTR", "BtnTL2", "BtnTR2", "BtnSelect", "BtnStart", "BtnMode",
NULL, NULL, NULL,
"ToolPen", "ToolRubber", "ToolBrush", "ToolPencil", "ToolAirbrush", "ToolFinger", "ToolMouse", "ToolLens", NULL, NULL,
"Touch", "Stylus", "Stylus2" };

char *absval[5] = { "Value", "Min  ", "Max  ", "Fuzz ", "Flat " };
char *relatives[REL_MAX + 1] = { "X", "Y", "Z", NULL, NULL, NULL, "HWheel", "Dial", "Wheel" };
char *absolutes[ABS_MAX + 1] = { "X", "Y", "Z", "Rx", "Ry", "Rz", "Throttle", "Rudder", "Wheel", "Gas", "Brake",
NULL, NULL, NULL, NULL, NULL,
"Hat0X", "Hat0Y", "Hat1X", "Hat1Y", "Hat2X", "Hat2Y", "Hat3X", "Hat 3Y", "Pressure", "Distance", "XTilt", "YTilt"};
char *leds[LED_MAX + 1] = { "NumLock", "CapsLock", "ScrollLock", "Compose", "Kana", "Sleep", "Suspend", "Mute" };
char *repeats[REP_MAX + 1] = { "Delay", "Period" };
char *sounds[SND_MAX + 1] = { "Bell", "Click" };

char **names[EV_MAX + 1] = { events, keys, relatives, absolutes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
NULL, NULL, leds, sounds, repeats };

#define BITS_PER_LONG (sizeof(long) * 8)
#define NBITS(x) ((((x)-1)/BITS_PER_LONG)+1)
#define OFF(x)  ((x)%BITS_PER_LONG)
#define BIT(x)  (1UL<<OFF(x))
#define LONG(x) ((x)/BITS_PER_LONG)
#define test_bit(bit, array)    ((array[LONG(bit)] >> OFF(bit)) & 1)

unsigned long bit[EV_MAX][NBITS(KEY_MAX)];

static gii_cmddata_getdevinfo devinfo =
{
  "Linux Evdev",  /* long device name */
  "edev",	  /* shorthand */
  emAll,	  /* can_generate */
  0,              /* num_buttons */
  0,              /* num_axes */
};

static gii_cmddata_getvalinfo valinfo[KEY_MAX];

static void init_devinfo(gii_input *inp)
{
  int i, n = 0;
  int abs[5];

  gii_levdev_priv *priv = inp->priv;

  ioctl(priv->fd, EVIOCGNAME(sizeof(devinfo.longname)), devinfo.longname);

  memset(bit, 0, sizeof(bit));
  ioctl(priv->fd, EVIOCGBIT(0, EV_MAX), bit[0]);
  /* key events */
  if (test_bit(1, bit[0]))
    {
      ioctl(priv->fd, EVIOCGBIT(1, KEY_MAX), bit[1]);
      for (i = 0; i < KEY_MAX; i++) 
	if (test_bit(i, bit[1])) n++;
    }
  devinfo.num_buttons = n;
  
  /* find highest indexed valuator */
  n = 0;
  if (test_bit(3, bit[0]))
    {
      ioctl(priv->fd, EVIOCGBIT(3, KEY_MAX), bit[3]);
      for (i = 0; i < KEY_MAX; i++) 
	if (test_bit(i, bit[3]))
	  n = n > i ? n : i;
    }
  /* FIXME:
   * this is actually not the number of axes, as the variable name suggests, 
   * but the highest index of a valuator
   * -stefan
   */
  devinfo.num_axes = n + 1;
  
  /* now find valuators */
  if (test_bit(EV_ABS, bit[0]))
    {
      ioctl(priv->fd, EVIOCGBIT(EV_ABS, KEY_MAX), bit[EV_ABS]);
      /* 	for (i = 0; i < KEY_MAX; i++)  */
      for (i = 0; i < KEY_MAX; i++) 
	if (test_bit(i, bit[EV_ABS]))
	  {
	    const char *name = names[EV_ABS] ? (names[EV_ABS][i] ? names[EV_ABS][i] : "?") : "?";
	    ioctl(priv->fd, EVIOCGABS(i), abs);
	    valinfo[i].number = i;
	    valinfo[i].range.min = abs[1];
	    valinfo[i].range.max = abs[2];
	    /* FIXME:
	     * no shortname is available for now, leave it out
	     * -stefan
	     */
	    strcpy(valinfo[i].longname, name);
	  }
    }
};

static void send_devinfo(gii_input *inp)
{
  gii_event ev;
  gii_cmddata_getdevinfo *dinfo;
  int size = sizeof(gii_cmd_nodata_event)+sizeof(gii_cmddata_getdevinfo);
  
  _giiEventBlank(&ev, size);
  
  ev.any.size   = size;
  ev.any.type   = evCommand;
  ev.any.origin = inp->origin;
  ev.cmd.code   = GII_CMDCODE_GETDEVINFO;
  
  dinfo = (gii_cmddata_getdevinfo *) ev.cmd.data;
  *dinfo = devinfo;
  
  _giiEvQueueAdd(inp, &ev);
}

static void send_valinfo(gii_input *inp, int n)
{
  gii_event ev;
  gii_cmddata_getvalinfo *vinfo;
  int size = sizeof(gii_cmd_nodata_event)+sizeof(gii_cmddata_getvalinfo);
  _giiEventBlank(&ev, size);
  
  ev.any.size   = size;
  ev.any.type   = evCommand;
  ev.any.origin = inp->origin;
  ev.cmd.code   = GII_CMDCODE_GETVALINFO;
  
  vinfo = (gii_cmddata_getvalinfo *) ev.cmd.data;
  *vinfo = valinfo[n];
  _giiEvQueueAdd(inp, &ev);
}


static int
GIIsendevent(gii_input *inp, gii_event *ev)
{
  if (ev->any.target != inp->origin &&
      ev->any.target != GII_EV_TARGET_ALL)
    return GGI_EEVNOTARGET;    /* not for us */

  if (ev->any.type != evCommand)
    return GGI_EEVUNKNOWN;
  
  if (ev->cmd.code == GII_CMDCODE_GETDEVINFO)
    {
      send_devinfo(inp);
      return 0;
    }
  
  if (ev->cmd.code == GII_CMDCODE_GETVALINFO)
    {
      gii_cmddata_getvalinfo *vi = (gii_cmddata_getvalinfo *) ev->cmd.data;
      if (vi->number == GII_VAL_QUERY_ALL)
	{
	  if (test_bit(3, bit[0]))
	    {
	      int i;
	      for (i = 0; i < KEY_MAX; i++) 
		if (test_bit(i, bit[3]))
		  send_valinfo(inp, i);
	    }
	  return 0;
	}

      send_valinfo(inp, vi->number);
      return 0;
    }
  
  return GGI_EEVUNKNOWN;	/* Unknown command */
}


static int
GIIclose(struct gii_input *inp)
{
  gii_levdev_priv *priv = inp->priv;
  
  if (priv->fd) close(priv->fd);
  free(priv);
  
  return 0;
}


int GIIdlinit(gii_input *inp, const char *args, void *argptr)
{
  const char *devname = "/dev/input/event0";
  gii_levdev_priv *priv;
  int fd;
  
  GIIDPRINT_LIBS("Linux evdev starting.\n");
  
  if (args && *args) devname = args;
  
  fd = open(devname, O_RDONLY);
  if (fd < 0) return GGI_ENODEVICE;
  
  priv = malloc(sizeof(*priv));
  if (priv == NULL)
    {
      close(fd);
      return GGI_ENOMEM;
    }

  if(_giiRegisterDevice(inp,&devinfo,valinfo)==0) {
    free(priv);
    close(fd);
    return GGI_ENOMEM;
  }

  
  inp->GIIsendevent = GIIsendevent;
  inp->GIIeventpoll = GII_levdev_poll;
  inp->GIIclose = GIIclose;
  
  inp->targetcan = emAll;
  inp->curreventmask = emAll;
  
  inp->maxfd = fd + 1;
  FD_SET(fd, &inp->fdset);
  
  priv->fd = fd;
  priv->eof = 0;
  
  inp->priv = priv;
  
  init_devinfo(inp);
  send_devinfo(inp);
  
  GIIDPRINT_LIBS("Linux evdev up.\n");
  
  return 0;
}
